@@ -1,5 +1,5 @@
 //
-// "$Id: howto-add_fd-and-popen.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
+// "$Id: howto-add_fd-and-popen.cxx 11870 2016-08-11 12:37:30Z AlbrechtS $"
 //
 //     How to use popen() and Fl::add_fd() - erco 10/04/04
 //     Originally from erco's cheat sheet, permission by author.
@@ -44,7 +44,9 @@
 FILE *G_fp = NULL;
 
 // Handler for add_fd() -- called whenever the ping command outputs a new line of data
-void HandleFD(int fd, void *data) {
+// Note: FL_SOCKET as 1st argument is used to fix a compiler error(!) on Windows 64-bit.
+// Unfortunately we need this in FLTK 1.3 - should hopefully be fixed in 1.4 with a better solution.
+void HandleFD(FL_SOCKET fd, void *data) {
   Fl_Multi_Browser *brow = (Fl_Multi_Browser*)data;
   char s[1024];
   if ( fgets(s, 1023, G_fp) == NULL ) {		// read the line of data
@@ -70,5 +72,5 @@ int main(int argc, char *argv[]) {
 }
 
 //
-// End of "$Id: howto-add_fd-and-popen.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
+// End of "$Id: howto-add_fd-and-popen.cxx 11870 2016-08-11 12:37:30Z AlbrechtS $".
 //
