@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Browser.cxx 9637 2012-07-24 04:37:22Z matt $"
+// "$Id: Fl_Browser.cxx 11464 2016-03-29 11:34:10Z AlbrechtS $"
 //
 // Browser widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2010 by Bill Spitzak and others.
+// Copyright 1998-2016 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
@@ -80,7 +80,7 @@ void* Fl_Browser::item_next(void* item) const {return ((FL_BLINE*)item)->next;}
 /**
   Returns the previous item before \p item.
   \param[in] item The 'current' item
-  \returns The previous item before \p item, or NULL if there none before this one.
+  \returns The previous item before \p item, or NULL if there are none before this one.
   \see item_first(), item_last(), item_next(), item_prev()
 */
 void* Fl_Browser::item_prev(void* item) const {return ((FL_BLINE*)item)->prev;}
@@ -284,6 +284,7 @@ void Fl_Browser::insert(int line, FL_BLINE* item) {
   \param[in] d Optional pointer to user data to be associated with the new line.
 */
 void Fl_Browser::insert(int line, const char* newtext, void* d) {
+  if (!newtext) newtext = "";		// STR #3269
   int l = (int) strlen(newtext);
   FL_BLINE* t = (FL_BLINE*)malloc(sizeof(FL_BLINE)+l);
   t->length = (short)l;
@@ -319,6 +320,7 @@ void Fl_Browser::move(int to, int from) {
 void Fl_Browser::text(int line, const char* newtext) {
   if (line < 1 || line > lines) return;
   FL_BLINE* t = find_line(line);
+  if (!newtext) newtext = "";		// STR #3269
   int l = (int) strlen(newtext);
   if (l > t->length) {
     FL_BLINE* n = (FL_BLINE*)malloc(sizeof(FL_BLINE)+l);
@@ -648,6 +650,34 @@ int Fl_Browser::topline() const {
   return lineno(top());
 }
 
+/**
+  Sets the default text size (in pixels) for the lines in the browser to \p newSize.
+
+  This method recalculates all item heights and caches the total height
+  internally for optimization of later item changes. This can be slow
+  if there are many items in the browser.
+
+  It returns immediately (w/o recalculation) if \p newSize equals
+  the current textsize().
+
+  You \e may need to call redraw() to see the effect and to have the
+  scrollbar positions recalculated.
+
+  You should set the text size \e before populating the browser with items
+  unless you really need to \e change the size later.
+*/
+void Fl_Browser::textsize(Fl_Fontsize newSize) {
+  if (newSize == textsize())
+    return; // avoid recalculation
+  Fl_Browser_::textsize(newSize);
+  new_list();
+  full_height_ = 0;
+  if (lines == 0) return;
+  for (FL_BLINE* itm=(FL_BLINE *)item_first(); itm; itm=(FL_BLINE *)item_next(itm)) {
+    full_height_ += item_height(itm);
+  }
+}
+
 /**
   Removes all the lines in the browser.
   \see add(), insert(), remove(), swap(int,int), clear()
@@ -789,7 +819,7 @@ int Fl_Browser::visible(int line) const {
 }
 
 /**
-  Returns the line number of the currently selected line, or 0 if none.
+  Returns the line number of the currently selected line, or 0 if none selected.
   \returns The line number of current selection, or 0 if none selected.
   \see select(), selected(), value(), item_select(), item_selected()
 */
@@ -930,5 +960,5 @@ Fl_Select_Browser::Fl_Select_Browser(int X,int Y,int W,int H,const char *L)
 
 
 //
-// End of "$Id: Fl_Browser.cxx 9637 2012-07-24 04:37:22Z matt $".
+// End of "$Id: Fl_Browser.cxx 11464 2016-03-29 11:34:10Z AlbrechtS $".
 //
