@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Choice.cxx 6872 2009-09-16 07:04:37Z AlbrechtS $"
+// "$Id: Fl_Choice.cxx 11907 2016-08-30 19:08:36Z greg.ercolano $"
 //
 // Choice widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2015 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -35,21 +26,34 @@
 // button: it draws the text of the current pick and a down-arrow.
 
 void Fl_Choice::draw() {
-  int dx = Fl::box_dx(FL_DOWN_BOX);
-  int dy = Fl::box_dy(FL_DOWN_BOX);
+  Fl_Boxtype btype = Fl::scheme() ? FL_UP_BOX		// non-default uses up box
+                                  : FL_DOWN_BOX;	// default scheme uses down box
+  int dx = Fl::box_dx(btype);
+  int dy = Fl::box_dy(btype);
+
+  // Arrow area
   int H = h() - 2 * dy;
-  int W = (H > 20) ? 20 : H;
+  int W = Fl::is_scheme("gtk+")    ? 20 :			// gtk+  -- fixed size
+          Fl::is_scheme("gleam")   ? 20 :			// gleam -- fixed size
+          Fl::is_scheme("plastic") ? ((H > 20) ? 20 : H)	// plastic: shrink if H<20
+                                   : ((H > 20) ? 20 : H);	// default: shrink if H<20
   int X = x() + w() - W - dx;
   int Y = y() + dy;
+
+  // Arrow object
   int w1 = (W - 4) / 3; if (w1 < 1) w1 = 1;
   int x1 = X + (W - 2 * w1 - 1) / 2;
   int y1 = Y + (H - w1 - 1) / 2;
 
   if (Fl::scheme()) {
-    draw_box(FL_UP_BOX, color());
+    // NON-DEFAULT SCHEME
 
+    // Draw widget box
+    draw_box(btype, color());
+
+    // Draw arrow area
     fl_color(active_r() ? labelcolor() : fl_inactive(labelcolor()));
-    if (!strcmp(Fl::scheme(), "plastic")) {
+    if (Fl::is_scheme("plastic")) {
       // Show larger up/down arrows...
       fl_polygon(x1, y1 + 3, x1 + w1, y1 + w1 + 3, x1 + 2 * w1, y1 + 3);
       fl_polygon(x1, y1 + 1, x1 + w1, y1 - w1 + 1, x1 + 2 * w1, y1 + 1);
@@ -67,27 +71,31 @@ void Fl_Choice::draw() {
       fl_yxline(x1 - 6, y1 - 8, y1 + 8);
     }
   } else {
-    if (fl_contrast(textcolor(), FL_BACKGROUND2_COLOR) == textcolor()) {
-      draw_box(FL_DOWN_BOX, FL_BACKGROUND2_COLOR);
-    } else {
-      draw_box(FL_DOWN_BOX, fl_lighter(color()));
-    }
-    draw_box(FL_UP_BOX,X,Y,W,H,color());
+    // DEFAULT SCHEME
 
+    // Draw widget box
+    if (fl_contrast(textcolor(), FL_BACKGROUND2_COLOR) == textcolor()) {
+      draw_box(btype, FL_BACKGROUND2_COLOR);
+    } else {
+      draw_box(btype, fl_lighter(color()));
+    }
+
+    // Draw arrow area
+    draw_box(FL_UP_BOX,X,Y,W,H,color());
     fl_color(active_r() ? labelcolor() : fl_inactive(labelcolor()));
     fl_polygon(x1, y1, x1 + w1, y1 + w1, x1 + 2 * w1, y1);
   }
 
   W += 2 * dx;
 
+  // Draw menu item's label
   if (mvalue()) {
     Fl_Menu_Item m = *mvalue();
     if (active_r()) m.activate(); else m.deactivate();
 
-    // ERCO
+    // Clip
     int xx = x() + dx, yy = y() + dy + 1, ww = w() - W, hh = H - 2;
-
-    fl_clip(xx, yy, ww, hh);
+    fl_push_clip(xx, yy, ww, hh);
 
     if ( Fl::scheme()) {
       Fl_Label l;
@@ -95,7 +103,7 @@ void Fl_Choice::draw() {
       l.image = 0;
       l.deimage = 0;
       l.type = m.labeltype_;
-      l.font = m.labelsize_ || m.labelfont_ ? m.labelfont_ : uchar(textfont());
+      l.font = m.labelsize_ || m.labelfont_ ? m.labelfont_ : textfont();
       l.size = m.labelsize_ ? m.labelsize_ : textsize();
       l.color= m.labelcolor_ ? m.labelcolor_ : textcolor();
       if (!m.active()) l.color = fl_inactive((Fl_Color)l.color);
@@ -113,11 +121,22 @@ void Fl_Choice::draw() {
     fl_pop_clip();
   }
 
+  // Widget's label
   draw_label();
 }
 
-Fl_Choice::Fl_Choice(int X, int Y, int W, int H, const char *l)
-: Fl_Menu_(X,Y,W,H,l) {
+/**
+  Create a new Fl_Choice widget using the given position, size and label string.
+  The default boxtype is \c FL_UP_BOX.
+
+  The constructor sets menu() to NULL.
+  See Fl_Menu_ for the methods to set or change the menu.
+
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Choice::Fl_Choice(int X, int Y, int W, int H, const char *L)
+: Fl_Menu_(X,Y,W,H,L) {
   align(FL_ALIGN_LEFT);
   when(FL_WHEN_RELEASE);
   textfont(FL_HELVETICA);
@@ -125,12 +144,24 @@ Fl_Choice::Fl_Choice(int X, int Y, int W, int H, const char *l)
   down_box(FL_BORDER_BOX);
 }
 
+/**
+  Sets the currently selected value using a pointer to menu item.
+  Changing the selected value causes a redraw().
+  \param[in] v pointer to menu item in the menu item array.
+  \returns non-zero if the new value is different to the old one.
+ */
 int Fl_Choice::value(const Fl_Menu_Item *v) {
   if (!Fl_Menu_::value(v)) return 0;
   redraw();
   return 1;
 }
 
+/**
+  Sets the currently selected value using the index into the menu item array.
+  Changing the selected value causes a redraw().
+  \param[in] v index of value in the menu item array.
+  \returns non-zero if the new value is different to the old one.
+ */
 int Fl_Choice::value(int v) {
   if (v == -1) return value((const Fl_Menu_Item *)0);
   if (v < 0 || v >= (size() - 1)) return 0;
@@ -187,5 +218,5 @@ int Fl_Choice::handle(int e) {
 }
 
 //
-// End of "$Id: Fl_Choice.cxx 6872 2009-09-16 07:04:37Z AlbrechtS $".
+// End of "$Id: Fl_Choice.cxx 11907 2016-08-30 19:08:36Z greg.ercolano $".
 //
