@@ -1,26 +1,17 @@
 //
-// "$Id: Fl_File_Icon2.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: Fl_File_Icon2.cxx 12028 2016-10-14 16:35:44Z AlbrechtS $"
 //
 // Fl_File_Icon system icon routines.
 //
 // KDE icon code donated by Maarten De Boer.
 //
-// Copyright 1999-2005 by Michael Sweet.
+// Copyright 1999-2010 by Michael Sweet.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -44,6 +35,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <FL/fl_utf8.h>
 #include "flstring.h"
 #include <ctype.h>
 #include <errno.h>
@@ -97,10 +89,10 @@ static char	*get_kde_val(char *str, const char *key);
 static const char *kdedir = NULL;
 
 
-//
-// 'Fl_File_Icon::load()' - Load an icon file...
-//
-
+/**
+  Loads the specified icon image. The format is deduced from the filename.
+  \param[in] f filename
+*/
 void
 Fl_File_Icon::load(const char *f)	// I - File to read from
 {
@@ -123,10 +115,11 @@ Fl_File_Icon::load(const char *f)	// I - File to read from
 }
 
 
-//
-// 'Fl_File_Icon::load_fti()' - Load an SGI-format FTI file...
-//
-
+/**
+  Loads an SGI icon file.
+  \param[in] fti icon filename
+  \return 0 on success, non-zero on error
+*/
 int					// O - 0 on success, non-zero on error
 Fl_File_Icon::load_fti(const char *fti)	// I - File to read from
 {
@@ -139,7 +132,7 @@ Fl_File_Icon::load_fti(const char *fti)	// I - File to read from
 
 
   // Try to open the file...
-  if ((fp = fopen(fti, "rb")) == NULL)
+  if ((fp = fl_fopen(fti, "rb")) == NULL)
   {
     Fl::error("Fl_File_Icon::load_fti(): Unable to open \"%s\" - %s",
               fti, strerror(errno));
@@ -340,12 +333,12 @@ Fl_File_Icon::load_fti(const char *fti)	// I - File to read from
 }
 
 
-//
-// 'Fl_File_Icon::load_image()' - Load an image icon file...
-//
-
-int					// O - 0 on success, non-0 on error
-Fl_File_Icon::load_image(const char *ifile)	// I - File to read from
+/**
+  Load an image icon file from an image filename.
+  \param[in] ifile image filename
+  \return 0 on success, non-zero on error
+*/
+int Fl_File_Icon::load_image(const char *ifile)	// I - File to read from
 {
   Fl_Shared_Image	*img;		// Image file
 
@@ -360,9 +353,10 @@ Fl_File_Icon::load_image(const char *ifile)	// I - File to read from
 		temp;		// Temporary color
     const uchar *row;		// Pointer into image
 
+    const int extra_data = img->ld() ? (img->ld()-img->w()*img->d()) : 0;
 
     // Loop through grayscale or RGB image...
-    for (y = 0, row = (const uchar *)(*(img->data())); y < img->h(); y ++, row += img->ld())
+    for (y = 0, row = (const uchar *)(*(img->data())); y < img->h(); y ++, row += extra_data)
     {
       for (x = 0, startx = 0, c = (Fl_Color)-1;
            x < img->w();
@@ -434,7 +428,6 @@ Fl_File_Icon::load_image(const char *ifile)	// I - File to read from
     int		x, y;			// X & Y in image
     int		startx;			// Starting X coord
 
-
     // Get the pixmap data...
     ptr = img->data();
     sscanf(*ptr, "%*d%*d%d%d", &ncolors, &chars_per_color);
@@ -577,24 +570,34 @@ Fl_File_Icon::load_image(const char *ifile)	// I - File to read from
   img->release();
 
 #ifdef DEBUG
-  printf("Icon File \"%s\":\n", xpm);
+{
+  int i;
+  printf("Icon File \"%s\":\n", ifile);
   for (i = 0; i < num_data_; i ++)
+  {
     printf("    %d,\n", data_[i]);
+  }
+}
 #endif // DEBUG
 
   return 0;
 }
 
 
-//
-// 'Fl_File_Icon::load_system_icons()' - Load the standard system icons/filetypes.
+/**
+  Loads all system-defined icons. This call is useful when using the
+  FileChooser widget and should be used when the application starts:
 
+  \code
+  Fl_File_Icon::load_system_icons();
+  \endcode
+*/
 void
 Fl_File_Icon::load_system_icons(void) {
   int		i;		// Looping var
   Fl_File_Icon	*icon;		// New icons
-  char		filename[1024];	// Filename
-  char		icondir[1024];	// Icon directory
+  char		filename[FL_PATH_MAX];	// Filename
+  char		icondir[FL_PATH_MAX];	// Icon directory
   static int	init = 0;	// Have the icons been initialized?
   const char * const icondirs[] = {
 		  "Bluecurve",	// Icon directories to look for, in order
@@ -618,7 +621,7 @@ Fl_File_Icon::load_system_icons(void) {
 		  VERTEX, 7000, 5000, END, LINE, VERTEX, 3000, 4000,
 		  VERTEX, 7000, 4000, END, LINE, VERTEX, 3000, 3000,
 		  VERTEX, 7000, 3000, END, LINE, VERTEX, 3000, 2000,
-		  VERTEX, 7000, 2000, END, 
+		  VERTEX, 7000, 2000, END,
 		  END
 		};
   static short	image[] = {	// Image file icon
@@ -804,7 +807,7 @@ load_kde_icons(const char *directory,	// I - Directory to load
   int		i;			// Looping var
   int		n;			// Number of entries in directory
   dirent	**entries;		// Entries in directory
-  char		full[1024];		// Full name of file
+  char		full[FL_PATH_MAX];	// Full name of file
 
 
   entries = (dirent **)0;
@@ -834,11 +837,11 @@ load_kde_mimelnk(const char *filename,	// I - mimelnk filename
                  const char *icondir) {	// I - Location of icons
   FILE		*fp;
   char		tmp[1024];
-  char		iconfilename[1024];
+  char		iconfilename[FL_PATH_MAX];
   char		pattern[1024];
   char		mimetype[1024];
   char		*val;
-  char		full_iconfilename[1024];
+  char		full_iconfilename[FL_PATH_MAX];
   Fl_File_Icon	*icon;
 
 
@@ -846,7 +849,7 @@ load_kde_mimelnk(const char *filename,	// I - mimelnk filename
   pattern[0]      = '\0';
   iconfilename[0] = '\0';
 
-  if ((fp = fopen(filename, "rb")) != NULL) {
+  if ((fp = fl_fopen(filename, "rb")) != NULL) {
     while (fgets(tmp, sizeof(tmp), fp)) {
       if ((val = get_kde_val(tmp, "Icon")) != NULL)
 	strlcpy(iconfilename, val, sizeof(iconfilename));
@@ -1011,5 +1014,5 @@ get_kde_val(char       *str,
 
 
 //
-// End of "$Id: Fl_File_Icon2.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: Fl_File_Icon2.cxx 12028 2016-10-14 16:35:44Z AlbrechtS $".
 //
