@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Repeat_Button.cxx 5219 2006-06-21 06:52:10Z matt $"
+// "$Id: Fl_Repeat_Button.cxx 9637 2012-07-24 04:37:22Z matt $"
 //
 // Repeat button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -65,6 +56,13 @@ int Fl_Repeat_Button::handle(int event) {
   }
 }
 
+
+Fl_Repeat_Button::Fl_Repeat_Button(int X,int Y,int W,int H,const char *l)
+: Fl_Button(X,Y,W,H,l) 
+{
+}
+
+
 //
-// End of "$Id: Fl_Repeat_Button.cxx 5219 2006-06-21 06:52:10Z matt $".
+// End of "$Id: Fl_Repeat_Button.cxx 9637 2012-07-24 04:37:22Z matt $".
 //
