@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Scroll.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
+// "$Id: Fl_Scroll.cxx 10776 2015-06-28 13:05:29Z AlbrechtS $"
 //
 // Scroll widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2011 by Bill Spitzak and others.
+// Copyright 1998-2015 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
@@ -35,7 +35,7 @@ void Fl_Scroll::clear() {
   add(scrollbar);
 }
 
-/** Insure the scrollbars are the last children */
+/** Insure the scrollbars are the last children. */
 void Fl_Scroll::fix_scrollbar_order() {
   Fl_Widget** a = (Fl_Widget**)array();
   if (a[children()-1] != &scrollbar) {
@@ -90,70 +90,73 @@ void Fl_Scroll::draw_clip(void* v,int X, int Y, int W, int H) {
 }
 
 /**
-   Calculate visibility/size/position of scrollbars, find children's bounding box.
-   The \p si paramater will be filled with data from the calculations.
-   Derived classes can make use of this call to figure out the scrolling area
-   eg. during resize() handling.
-   \param[in] si -- ScrollInfo structure
-   \returns Structure containing the calculated info.
+  Calculate visibility/size/position of scrollbars, find children's bounding box.
+
+  The \p si parameter will be filled with data from the calculations.
+  Derived classes can make use of this call to figure out the scrolling area
+  eg. during resize() handling.
+
+  \param[in] si -- ScrollInfo structure
+  \returns Structure containing the calculated info.
 */
 void Fl_Scroll::recalc_scrollbars(ScrollInfo &si) {
 
   // inner box of widget (excluding scrollbars)
-  si.innerbox_x = x()+Fl::box_dx(box());
-  si.innerbox_y = y()+Fl::box_dy(box());
-  si.innerbox_w = w()-Fl::box_dw(box());
-  si.innerbox_h = h()-Fl::box_dh(box());
+  si.innerbox.x = x()+Fl::box_dx(box());
+  si.innerbox.y = y()+Fl::box_dy(box());
+  si.innerbox.w = w()-Fl::box_dw(box());
+  si.innerbox.h = h()-Fl::box_dh(box());
 
   // accumulate a bounding box for all the children
-  si.child_l = si.innerbox_x;
-  si.child_r = si.innerbox_x;
-  si.child_b = si.innerbox_y;
-  si.child_t = si.innerbox_y;
+  si.child.l = si.innerbox.x;
+  si.child.r = si.innerbox.x;
+  si.child.b = si.innerbox.y;
+  si.child.t = si.innerbox.y;
   int first = 1;
   Fl_Widget*const* a = array();
-  for (int i=children()-2; i--;) {
+  for (int i=children(); i--;) {
     Fl_Widget* o = *a++;
+    if ( o==&scrollbar || o==&hscrollbar ) continue;
     if ( first ) {
         first = 0;
-	si.child_l = o->x();
-	si.child_r = o->x()+o->w();
-	si.child_b = o->y()+o->h();
-	si.child_t = o->y();
+	si.child.l = o->x();
+	si.child.r = o->x()+o->w();
+	si.child.b = o->y()+o->h();
+	si.child.t = o->y();
     } else {
-	if (o->x() < si.child_l) si.child_l = o->x();
-	if (o->y() < si.child_t) si.child_t = o->y();
-	if (o->x()+o->w() > si.child_r) si.child_r = o->x()+o->w();
-	if (o->y()+o->h() > si.child_b) si.child_b = o->y()+o->h();
+	if (o->x() < si.child.l) si.child.l = o->x();
+	if (o->y() < si.child.t) si.child.t = o->y();
+	if (o->x()+o->w() > si.child.r) si.child.r = o->x()+o->w();
+	if (o->y()+o->h() > si.child.b) si.child.b = o->y()+o->h();
     }
   }
 
   // Turn the scrollbars on and off as necessary.
   // See if children would fit if we had no scrollbars...
   {
-    int X = si.innerbox_x;
-    int Y = si.innerbox_y;
-    int W = si.innerbox_w;
-    int H = si.innerbox_h;
+    int X = si.innerbox.x;
+    int Y = si.innerbox.y;
+    int W = si.innerbox.w;
+    int H = si.innerbox.h;
 
     si.scrollsize = scrollbar_size_ ? scrollbar_size_ : Fl::scrollbar_size();
     si.vneeded = 0;
     si.hneeded = 0;
     if (type() & VERTICAL) {
-      if ((type() & ALWAYS_ON) || si.child_t < Y || si.child_b > Y+H) {
+      if ((type() & ALWAYS_ON) || si.child.t < Y || si.child.b > Y+H) {
 	si.vneeded = 1;
 	W -= si.scrollsize;
 	if (scrollbar.align() & FL_ALIGN_LEFT) X += si.scrollsize;
       }
     }
     if (type() & HORIZONTAL) {
-      if ((type() & ALWAYS_ON) || si.child_l < X || si.child_r > X+W) {
+      if ((type() & ALWAYS_ON) || si.child.l < X || si.child.r > X+W) {
 	si.hneeded = 1;
 	H -= si.scrollsize;
 	if (scrollbar.align() & FL_ALIGN_TOP) Y += si.scrollsize;
 	// recheck vertical since we added a horizontal scrollbar
 	if (!si.vneeded && (type() & VERTICAL)) {
-	  if ((type() & ALWAYS_ON) || si.child_t < Y || si.child_b > Y+H) {
+	  if ((type() & ALWAYS_ON) || si.child.t < Y || si.child.b > Y+H) {
 	    si.vneeded = 1;
 	    W -= si.scrollsize;
 	    if (scrollbar.align() & FL_ALIGN_LEFT) X += si.scrollsize;
@@ -161,61 +164,61 @@ void Fl_Scroll::recalc_scrollbars(ScrollInfo &si) {
 	}
       }
     }
-    si.innerchild_x = X;
-    si.innerchild_y = Y;
-    si.innerchild_w = W;
-    si.innerchild_h = H;
+    si.innerchild.x = X;
+    si.innerchild.y = Y;
+    si.innerchild.w = W;
+    si.innerchild.h = H;
   }
 
   // calculate hor scrollbar position
-  si.hscroll_x = si.innerchild_x;
-  si.hscroll_y = (scrollbar.align() & FL_ALIGN_TOP) 
-		     ? si.innerbox_y
-		     : si.innerbox_y + si.innerbox_h - si.scrollsize;
-  si.hscroll_w = si.innerchild_w;
-  si.hscroll_h = si.scrollsize;
+  si.hscroll.x = si.innerchild.x;
+  si.hscroll.y = (scrollbar.align() & FL_ALIGN_TOP)
+		     ? si.innerbox.y
+		     : si.innerbox.y + si.innerbox.h - si.scrollsize;
+  si.hscroll.w = si.innerchild.w;
+  si.hscroll.h = si.scrollsize;
 
   // calculate ver scrollbar position
-  si.vscroll_x = (scrollbar.align() & FL_ALIGN_LEFT)
-                     ? si.innerbox_x
-		     : si.innerbox_x + si.innerbox_w - si.scrollsize;
-  si.vscroll_y = si.innerchild_y;
-  si.vscroll_w = si.scrollsize;
-  si.vscroll_h = si.innerchild_h;
+  si.vscroll.x = (scrollbar.align() & FL_ALIGN_LEFT)
+                     ? si.innerbox.x
+		     : si.innerbox.x + si.innerbox.w - si.scrollsize;
+  si.vscroll.y = si.innerchild.y;
+  si.vscroll.w = si.scrollsize;
+  si.vscroll.h = si.innerchild.h;
 
   // calculate h/v scrollbar values (pos/size/first/total)
-  si.hpos = si.innerchild_x - si.child_l;
-  si.hsize = si.innerchild_w;
-  si.hfirst = 0;
-  si.htotal = si.child_r - si.child_l;
-  if ( si.hpos < 0 ) { si.htotal += (-si.hpos); si.hfirst = si.hpos; }
+  si.hscroll.pos = si.innerchild.x - si.child.l;
+  si.hscroll.size = si.innerchild.w;
+  si.hscroll.first = 0;
+  si.hscroll.total = si.child.r - si.child.l;
+  if ( si.hscroll.pos < 0 ) { si.hscroll.total += (-si.hscroll.pos); si.hscroll.first = si.hscroll.pos; }
 
-  si.vpos = si.innerchild_y - si.child_t;
-  si.vsize = si.innerchild_h;
-  si.vfirst = 0;
-  si.vtotal = si.child_b - si.child_t;
-  if ( si.vpos < 0 ) { si.vtotal += (-si.vpos); si.vfirst = si.vpos; }
+  si.vscroll.pos = si.innerchild.y - si.child.t;
+  si.vscroll.size = si.innerchild.h;
+  si.vscroll.first = 0;
+  si.vscroll.total = si.child.b - si.child.t;
+  if ( si.vscroll.pos < 0 ) { si.vscroll.total += (-si.vscroll.pos); si.vscroll.first = si.vscroll.pos; }
 
 //  printf("DEBUG --- ScrollInfo ---\n");
 //  printf("DEBUG        scrollsize: %d\n", si.scrollsize);
 //  printf("DEBUG  hneeded, vneeded: %d %d\n", si.hneeded, si.vneeded);
-//  printf("DEBUG     innerbox xywh: %d %d %d %d\n", si.innerbox_x,   si.innerbox_y,   si.innerbox_w,   si.innerbox_h);
-//  printf("DEBUG   innerchild xywh: %d %d %d %d\n", si.innerchild_x, si.innerchild_y, si.innerchild_w, si.innerchild_h);
-//  printf("DEBUG        child lrbt: %d %d %d %d\n", si.child_l, si.child_r, si.child_b, si.child_t);
-//  printf("DEBUG      hscroll xywh: %d %d %d %d\n", si.hscroll_x, si.hscroll_y, si.hscroll_w, si.hscroll_h);
-//  printf("DEBUG      vscroll xywh: %d %d %d %d\n", si.vscroll_x, si.vscroll_y, si.vscroll_w, si.vscroll_h);
-//  printf("DEBUG  horz scroll vals: %d %d %d %d\n", si.hpos, si.hsize, si.hfirst, si.htotal);
-//  printf("DEBUG  vert scroll vals: %d %d %d %d\n", si.vpos, si.vsize, si.vfirst, si.vtotal);
+//  printf("DEBUG     innerbox.x, si.innerbox.y, si.innerbox.w,si.innerbox.h);
+//  printf("DEBUG   innerchild.xywh: %d %d %d %d\n", si.innerchild.x, si.innerchild.y, si.innerchild.w, si.innerchild.h);
+//  printf("DEBUG        child lrbt: %d %d %d %d\n", si.child.l, si.child.r, si.child.b, si.child.t);
+//  printf("DEBUG      hscroll xywh: %d %d %d %d\n", si.hscroll.x, si.hscroll.y, si.hscroll.w, si.hscroll.h);
+//  printf("DEBUG      vscroll xywh: %d %d %d %d\n", si.vscroll.x, si.vscroll.y, si.vscroll.w, si.vscroll.h);
+//  printf("DEBUG  horz scroll vals: %d %d %d %d\n", si.hscroll.pos, si.hscroll.size, si.hscroll.first, si.hscroll.total);
+//  printf("DEBUG  vert scroll vals: %d %d %d %d\n", si.vscroll.pos, si.vscroll.size, si.vscroll.first, si.vscroll.total);
 //  printf("DEBUG \n");
 }
 
 /**
   Returns the bounding box for the interior of the scrolling area, inside
   the scrollbars.
-  
+
   Currently this is only reliable after draw(), and before any resizing of
   the Fl_Scroll or any child widgets occur.
-  
+
   \todo The visibility of the scrollbars ought to be checked/calculated
   outside of the draw() method (STR #1895).
 */
@@ -286,7 +289,7 @@ void Fl_Scroll::draw() {
       }
       else if (!si.vneeded && scrollbar.visible()) {
 	scrollbar.clear_visible();
-	draw_clip(this, si.vscroll_x, si.vscroll_y, si.vscroll_w, si.vscroll_h);
+	draw_clip(this, si.vscroll.x, si.vscroll.y, si.vscroll.w, si.vscroll.h);
 	d = FL_DAMAGE_ALL;
       }
       if (si.hneeded && !hscrollbar.visible()) {
@@ -295,7 +298,7 @@ void Fl_Scroll::draw() {
       }
       else if (!si.hneeded && hscrollbar.visible()) {
 	hscrollbar.clear_visible();
-	draw_clip(this, si.hscroll_x, si.hscroll_y, si.hscroll_w, si.hscroll_h);
+	draw_clip(this, si.hscroll.x, si.hscroll.y, si.hscroll.w, si.hscroll.h);
 	d = FL_DAMAGE_ALL;
       }
       else if ( hscrollbar.h() != si.scrollsize || scrollbar.w() != si.scrollsize ) {
@@ -303,13 +306,13 @@ void Fl_Scroll::draw() {
          d = FL_DAMAGE_ALL;
       }
 
-      scrollbar.resize(si.vscroll_x, si.vscroll_y, si.vscroll_w, si.vscroll_h);
-      oldy = yposition_ = si.vpos;	// si.innerchild_y - si.child_t;
-      scrollbar.value(si.vpos, si.vsize, si.vfirst, si.vtotal);
+      scrollbar.resize(si.vscroll.x, si.vscroll.y, si.vscroll.w, si.vscroll.h);
+      oldy = yposition_ = si.vscroll.pos;	// si.innerchild.y - si.child.t;
+      scrollbar.value(si.vscroll.pos, si.vscroll.size, si.vscroll.first, si.vscroll.total);
 
-      hscrollbar.resize(si.hscroll_x, si.hscroll_y, si.hscroll_w, si.hscroll_h);
-      oldx = xposition_ = si.hpos;	// si.innerchild_x - si.child_l;
-      hscrollbar.value(si.hpos, si.hsize, si.hfirst, si.htotal);
+      hscrollbar.resize(si.hscroll.x, si.hscroll.y, si.hscroll.w, si.hscroll.h);
+      oldx = xposition_ = si.hscroll.pos;	// si.innerchild.x - si.child.l;
+      hscrollbar.value(si.hscroll.pos, si.hscroll.size, si.hscroll.first, si.hscroll.total);
   }
 
   // draw the scrollbars:
@@ -327,6 +330,30 @@ void Fl_Scroll::draw() {
   }
 }
 
+/**
+  Resizes the Fl_Scroll widget and moves its children if necessary.
+
+  The Fl_Scroll widget first resizes itself, and then it moves all its
+  children if (and only if) the Fl_Scroll widget has been moved. The
+  children are moved by the same amount as the Fl_Scroll widget has been
+  moved, hence all children keep their relative positions.
+
+  \note Fl_Scroll::resize() does \b not call Fl_Group::resize(), and
+  child widgets are \b not resized.
+
+  Since children of an Fl_Scroll are not resized, the resizable() widget
+  is ignored (if it is set).
+
+  The scrollbars are moved to their proper positions, as given by
+  Fl_Scroll::scrollbar.align(), and switched on or off as necessary.
+
+  \note Due to current (FLTK 1.3.x) implementation constraints some of this
+  may effectively be postponed until the Fl_Scroll is drawn the next time.
+  This may change in a future release.
+
+  \sa Fl_Group::resizable()
+  \sa Fl_Widget::resize(int,int,int,int)
+*/
 void Fl_Scroll::resize(int X, int Y, int W, int H) {
   int dx = X-x(), dy = Y-y();
   int dw = W-w(), dh = H-h();
@@ -345,12 +372,27 @@ void Fl_Scroll::resize(int X, int Y, int W, int H) {
     scrollbar.position(al?X:X+W-scrollbar.w(), (at&&pad)?Y+hscrollbar.h():Y);
     hscrollbar.position((al&&pad)?X+scrollbar.w():X, at?Y:Y+H-hscrollbar.h());
   } else {
-    // FIXME recalculation of scrollbars needs to be moved out fo "draw()" (STR #1895)
+    // FIXME recalculation of scrollbars needs to be moved out of "draw()" (STR #1895)
     redraw(); // need full recalculation of scrollbars
   }
 }
 
-/**  Moves the contents of the scroll group to a new position.*/
+/**  Moves the contents of the scroll group to a new position.
+
+  This is like moving the scrollbars of the Fl_Scroll around. For instance:
+  \code
+    Fl_Scroll scroll (10,10,200,200);
+    Fl_Box b1 ( 10, 10,50,50,"b1"); // relative (x,y) = (0,0)
+    Fl_Box b2 ( 60, 60,50,50,"b2"); // relative (x,y) = (50,50)
+    Fl_Box b3 ( 60,110,50,50,"b3"); // relative (x,y) = (50,100)
+    // populate scroll with more children ...
+    scroll.end();
+    scroll.scroll_to(50,100);
+  \endcode
+  will move the logical origin of the internal scroll area to (-50,-100)
+  relative to the origin of the Fl_Scroll (10,10), i.e. Fl_Box b3 will
+  be visible in the top left corner of the scroll area.
+*/
 void Fl_Scroll::scroll_to(int X, int Y) {
   int dx = xposition_-X;
   int dy = yposition_-Y;
@@ -379,15 +421,16 @@ void Fl_Scroll::scrollbar_cb(Fl_Widget* o, void*) {
 /**
   Creates a new Fl_Scroll widget using the given position,
   size, and label string. The default boxtype is FL_NO_BOX.
-  <P>The destructor <I>also deletes all the children</I>. This allows a
+
+  The destructor <I>also deletes all the children</I>. This allows a
   whole tree to be deleted at once, without having to keep a pointer to
-  all the children in the user code. A kludge has been done so the 
+  all the children in the user code. A kludge has been done so the
   Fl_Scroll and all of its children can be automatic (local)
-  variables, but you must declare the Fl_Scroll<I>first</I>, so
+  variables, but you must declare the Fl_Scroll <I>first</I>, so
   that it is destroyed last.
 */
 Fl_Scroll::Fl_Scroll(int X,int Y,int W,int H,const char* L)
-  : Fl_Group(X,Y,W,H,L), 
+  : Fl_Group(X,Y,W,H,L),
     scrollbar(X+W-Fl::scrollbar_size(),Y,
               Fl::scrollbar_size(),H-Fl::scrollbar_size()),
     hscrollbar(X,Y+H-Fl::scrollbar_size(),
@@ -407,5 +450,5 @@ int Fl_Scroll::handle(int event) {
 }
 
 //
-// End of "$Id: Fl_Scroll.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
+// End of "$Id: Fl_Scroll.cxx 10776 2015-06-28 13:05:29Z AlbrechtS $".
 //
