@@ -1,24 +1,15 @@
 //
-// "$Id: filename_absolute.cxx 6644 2009-01-23 10:47:53Z AlbrechtS $"
+// "$Id: filename_absolute.cxx 11243 2016-02-27 15:14:42Z AlbrechtS $"
 //
 // Filename expansion routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -32,19 +23,14 @@
 */
 
 #include <FL/filename.H>
+#include <FL/fl_utf8.h>
 #include <stdlib.h>
 #include "flstring.h"
 #include <ctype.h>
 #if defined(WIN32) && !defined(__CYGWIN__)
 # include <direct.h>
-// Visual C++ 2005 incorrectly displays a warning about the use of POSIX APIs
-// on Windows, which is supposed to be POSIX compliant...
-#  define getcwd _getcwd
 #else
 #  include <unistd.h>
-#  ifdef __EMX__
-#    define getcwd _getcwd2
-#  endif
 #endif
 
 #if defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__)
@@ -53,6 +39,20 @@ inline int isdirsep(char c) {return c=='/' || c=='\\';}
 #define isdirsep(c) ((c)=='/')
 #endif
 
+/** Makes a filename absolute from a relative filename.
+    \code
+    #include <FL/filename.H>
+    [..]
+    chdir("/var/tmp");
+    fl_filename_absolute(out, sizeof(out), "foo.txt");         // out="/var/tmp/foo.txt"
+    fl_filename_absolute(out, sizeof(out), "./foo.txt");       // out="/var/tmp/foo.txt"
+    fl_filename_absolute(out, sizeof(out), "../log/messages"); // out="/var/log/messages"
+    \endcode
+    \param[out] to resulting absolute filename
+    \param[in]  tolen size of the absolute filename buffer 
+    \param[in]  from relative filename
+    \return 0 if no change, non zero otherwise
+ */
 int fl_filename_absolute(char *to, int tolen, const char *from) {
   if (isdirsep(*from) || *from == '|'
 #if defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__)
@@ -67,7 +67,7 @@ int fl_filename_absolute(char *to, int tolen, const char *from) {
   char *temp = new char[tolen];
   const char *start = from;
 
-  a = getcwd(temp, tolen);
+  a = fl_getcwd(temp, tolen);
   if (!a) {
     strlcpy(to, from, tolen);
     delete[] temp;
@@ -83,7 +83,7 @@ int fl_filename_absolute(char *to, int tolen, const char *from) {
   while (*start == '.') {
     if (start[1]=='.' && isdirsep(start[2])) {
       char *b;
-      for (b = a-1; b >= temp && !isdirsep(*b); b--);
+      for (b = a-1; b >= temp && !isdirsep(*b); b--) {/*empty*/}
       if (b < temp) break;
       a = b;
       start += 3;
@@ -106,20 +106,57 @@ int fl_filename_absolute(char *to, int tolen, const char *from) {
   return 1;
 }
 
-/*
- * 'fl_filename_relative()' - Make a filename relative to the current working directory.
+/** Makes a filename relative to the current working directory.
+    \code
+    #include <FL/filename.H>
+    [..]
+    chdir("/var/tmp/somedir");       // set cwd to /var/tmp/somedir
+    [..]
+    char out[FL_PATH_MAX];
+    fl_filename_relative(out, sizeof(out), "/var/tmp/somedir/foo.txt");  // out="foo.txt",    return=1
+    fl_filename_relative(out, sizeof(out), "/var/tmp/foo.txt");          // out="../foo.txt", return=1
+    fl_filename_relative(out, sizeof(out), "foo.txt");                   // out="foo.txt",    return=0 (no change)
+    fl_filename_relative(out, sizeof(out), "./foo.txt");                 // out="./foo.txt",  return=0 (no change)
+    fl_filename_relative(out, sizeof(out), "../foo.txt");                // out="../foo.txt", return=0 (no change)
+    \endcode
+    \param[out] to resulting relative filename
+    \param[in]  tolen size of the relative filename buffer 
+    \param[in]  from absolute filename
+    \return 0 if no change, non zero otherwise
  */
-
 int					// O - 0 if no change, 1 if changed
 fl_filename_relative(char       *to,	// O - Relative filename
                      int        tolen,	// I - Size of "to" buffer
-                     const char *from) {// I - Absolute filename
-  char		*newslash;		// Directory separator
+                     const char *from)  // I - Absolute filename
+{
+  char cwd_buf[FL_PATH_MAX];	// Current directory
+  // get the current directory and return if we can't
+  if (!fl_getcwd(cwd_buf, sizeof(cwd_buf))) {
+    strlcpy(to, from, tolen);
+    return 0;
+  }
+  return fl_filename_relative(to, tolen, from, cwd_buf);
+}
+
+
+/** Makes a filename relative to any other directory.
+ \param[out] to resulting relative filename
+ \param[in]  tolen size of the relative filename buffer 
+ \param[in]  from absolute filename
+ \param[in]  base relative to this absolute path
+ \return 0 if no change, non zero otherwise
+ */
+int					// O - 0 if no change, 1 if changed
+fl_filename_relative(char       *to,	// O - Relative filename
+                     int        tolen,	// I - Size of "to" buffer
+                     const char *from,  // I - Absolute filename
+                     const char *base) { // I - Find path relative to this path
+  
+  char          *newslash;		// Directory separator
   const char	*slash;			// Directory separator
-  char		cwd_buf[1024];		// Current directory
-  char          *cwd = cwd_buf;
-
-
+  char          *cwd = 0L, *cwd_buf = 0L;
+  if (base) cwd = cwd_buf = strdup(base);
+  
   // return if "from" is not an absolute path
 #if defined(WIN32) || defined(__EMX__)
   if (from[0] == '\0' ||
@@ -129,15 +166,23 @@ fl_filename_relative(char       *to,	// O - Relative filename
   if (from[0] == '\0' || !isdirsep(*from)) {
 #endif // WIN32 || __EMX__
     strlcpy(to, from, tolen);
+    if (cwd_buf) free(cwd_buf);
     return 0;
   }
-
-  // get the current directory and return if we can't
-  if (!getcwd(cwd_buf, sizeof(cwd_buf))) {
+        
+  // return if "cwd" is not an absolute path
+#if defined(WIN32) || defined(__EMX__)
+  if (!cwd || cwd[0] == '\0' ||
+      (!isdirsep(*cwd) && !isalpha(*cwd) && cwd[1] != ':' &&
+       !isdirsep(cwd[2]))) {
+#else
+  if (!cwd || cwd[0] == '\0' || !isdirsep(*cwd)) {
+#endif // WIN32 || __EMX__
     strlcpy(to, from, tolen);
+    if (cwd_buf) free(cwd_buf);
     return 0;
   }
-
+              
 #if defined(WIN32) || defined(__EMX__)
   // convert all backslashes into forward slashes
   for (newslash = strchr(cwd, '\\'); newslash; newslash = strchr(newslash + 1, '\\'))
@@ -146,12 +191,15 @@ fl_filename_relative(char       *to,	// O - Relative filename
   // test for the exact same string and return "." if so
   if (!strcasecmp(from, cwd)) {
     strlcpy(to, ".", tolen);
+    free(cwd_buf);
     return (1);
   }
 
   // test for the same drive. Return the absolute path if not
   if (tolower(*from & 255) != tolower(*cwd & 255)) {
+    // Not the same drive...
     strlcpy(to, from, tolen);
+    free(cwd_buf);
     return 0;
   }
 
@@ -161,6 +209,7 @@ fl_filename_relative(char       *to,	// O - Relative filename
   // test for the exact same string and return "." if so
   if (!strcmp(from, cwd)) {
     strlcpy(to, ".", tolen);
+    free(cwd_buf);
     return (1);
   }
 #endif // WIN32 || __EMX__
@@ -186,6 +235,7 @@ fl_filename_relative(char       *to,	// O - Relative filename
   if (isdirsep(*slash)) slash ++;
 
   // do the same for the current dir
+  if (isdirsep(*newslash)) newslash --;
   if (*newslash != '\0')
     while (!isdirsep(*newslash) && newslash > cwd) newslash --;
 
@@ -203,10 +253,11 @@ fl_filename_relative(char       *to,	// O - Relative filename
   // finally add the differing path from "from"
   strlcat(to, slash, tolen);
 
+  free(cwd_buf);
   return 1;
 }
 
 
 //
-// End of "$Id: filename_absolute.cxx 6644 2009-01-23 10:47:53Z AlbrechtS $".
+// End of "$Id: filename_absolute.cxx 11243 2016-02-27 15:14:42Z AlbrechtS $".
 //
