@@ -1,5 +1,5 @@
 //
-// "$Id: fl_gtk.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
+// "$Id: fl_gtk.cxx 10781 2015-07-09 00:10:44Z AlbrechtS $"
 //
 // "GTK" drawing routines for the Fast Light Tool Kit (FLTK).
 //
@@ -30,11 +30,9 @@ extern void fl_internal_boxtype(Fl_Boxtype, Fl_Box_Draw_F*);
 
 
 static void gtk_color(Fl_Color c) {
-  if (Fl::draw_box_active()) fl_color(c);
-  else fl_color(fl_inactive(c));
+  Fl::set_box_color(c);
 }
 
-
 static void gtk_up_frame(int x, int y, int w, int h, Fl_Color c) {
   gtk_color(fl_color_average(FL_WHITE, c, 0.5));
   fl_xyline(x + 2, y + 1, x + w - 3);
@@ -203,8 +201,8 @@ static void draw(int which, int x,int y,int w,int h, int inset)
   }
 }
 
-void gtk_round_up_box(int x, int y, int w, int h, Fl_Color c) {
-  fl_color(c);
+static void gtk_round_up_box(int x, int y, int w, int h, Fl_Color c) {
+  gtk_color(c);
   draw(FILL,	    x,   y, w,   h, 2);
 
   gtk_color(fl_color_average(FL_BLACK, c, 0.025f));
@@ -234,8 +232,8 @@ void gtk_round_up_box(int x, int y, int w, int h, Fl_Color c) {
   draw(CLOSED,	    x,   y, w,   h, 0);
 }
 
-void gtk_round_down_box(int x, int y, int w, int h, Fl_Color c) {
-  fl_color(c);
+static void gtk_round_down_box(int x, int y, int w, int h, Fl_Color c) {
+  gtk_color(c);
   draw(FILL,	    x,   y, w,   h, 2);
 
   gtk_color(fl_color_average(FL_BLACK, c, 0.05f));
@@ -291,5 +289,5 @@ Fl_Boxtype fl_define_FL_GTK_UP_BOX() {
 
 
 //
-// End of "$Id: fl_gtk.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
+// End of "$Id: fl_gtk.cxx 10781 2015-07-09 00:10:44Z AlbrechtS $".
 //
