@@ -1,24 +1,15 @@
 //
-// "$Id: fl_oval_box.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: fl_oval_box.cxx 10781 2015-07-09 00:10:44Z AlbrechtS $"
 //
 // Oval box drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2011 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -26,19 +17,19 @@
 //
 
 
-// Less-used box types are in seperate files so they are not linked
+// Less-used box types are in separate files so they are not linked
 // in if not used.
 
 #include <FL/Fl.H>
 #include <FL/fl_draw.H>
 
 static void fl_oval_flat_box(int x, int y, int w, int h, Fl_Color c) {
-  fl_color(c);
+  Fl::set_box_color(c);
   fl_pie(x, y, w, h, 0, 360);
 }
 
 static void fl_oval_frame(int x, int y, int w, int h, Fl_Color c) {
-  fl_color(c);
+  Fl::set_box_color(c);
   fl_arc(x, y, w, h, 0, 360);
 }
 
@@ -62,5 +53,5 @@ Fl_Boxtype fl_define_FL_OVAL_BOX() {
 }
 
 //
-// End of "$Id: fl_oval_box.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: fl_oval_box.cxx 10781 2015-07-09 00:10:44Z AlbrechtS $".
 //
