@@ -1,5 +1,5 @@
 //
-// "$Id: fl_scroll_area.cxx 10078 2014-01-22 20:39:21Z manolo $"
+// "$Id: fl_scroll_area.cxx 10560 2015-02-08 06:48:19Z manolo $"
 //
 // Scrolling routines for the Fast Light Tool Kit (FLTK).
 //
@@ -144,7 +144,7 @@ void fl_scroll(int X, int Y, int W, int H, int dx, int dy,
 #elif defined(__APPLE_QUARTZ__)
   CGImageRef img = Fl_X::CGImage_from_window_rect(Fl_Window::current(), src_x, src_y, src_w, src_h);
   if (img) {
-    CGRect rect = { { dest_x, dest_y }, { src_w, src_h } };
+    CGRect rect = CGRectMake(dest_x, dest_y, src_w, src_h);
     Fl_X::q_begin_image(rect, 0, 0, src_w, src_h);
     CGContextDrawImage(fl_gc, rect, img);
     Fl_X::q_end_image();
@@ -158,5 +158,5 @@ void fl_scroll(int X, int Y, int W, int H, int dx, int dy,
 }
 
 //
-// End of "$Id: fl_scroll_area.cxx 10078 2014-01-22 20:39:21Z manolo $".
+// End of "$Id: fl_scroll_area.cxx 10560 2015-02-08 06:48:19Z manolo $".
 //
