@@ -1,9 +1,9 @@
 /*
- * "$Id: flstring.h 10074 2014-01-21 11:07:43Z AlbrechtS $"
+ * "$Id: flstring.h 11094 2016-01-31 02:49:56Z AlbrechtS $"
  *
  * Common string header file for the Fast Light Tool Kit (FLTK).
  *
- * Copyright 1998-2010 by Bill Spitzak and others.
+ * Copyright 1998-2016 by Bill Spitzak and others.
  *
  * This library is free software. Distribution and use rights are outlined in
  * the file "COPYING" which should have been included with this file.  If this
@@ -60,12 +60,12 @@ extern "C" {
 #  endif /* __cplusplus */
 
 FL_EXPORT extern int fl_snprintf(char *, size_t, const char *, ...);
-#  if !HAVE_SNPRINTF
+#  ifndef HAVE_SNPRINTF
 #    define snprintf fl_snprintf
 #  endif /* !HAVE_SNPRINTF */
 
 FL_EXPORT extern int fl_vsnprintf(char *, size_t, const char *, va_list ap);
-#  if !HAVE_VSNPRINTF
+#  ifndef HAVE_VSNPRINTF
 #    define vsnprintf fl_vsnprintf
 #  endif /* !HAVE_VSNPRINTF */
 
@@ -75,12 +75,12 @@ FL_EXPORT extern int fl_vsnprintf(char *, size_t, const char *, va_list ap);
  */
 
 FL_EXPORT extern size_t fl_strlcat(char *, const char *, size_t);
-#  if !HAVE_STRLCAT
+#  ifndef HAVE_STRLCAT
 #    define strlcat fl_strlcat
 #  endif /* !HAVE_STRLCAT */
 
 FL_EXPORT extern size_t fl_strlcpy(char *, const char *, size_t);
-#  if !HAVE_STRLCPY
+#  ifndef HAVE_STRLCPY
 #    define strlcpy fl_strlcpy
 #  endif /* !HAVE_STRLCPY */
 
@@ -97,5 +97,5 @@ FL_EXPORT extern int fl_ascii_strcasecmp(const char *s, const char *t);
 #endif /* !flstring_h */
 
 /*
- * End of "$Id: flstring.h 10074 2014-01-21 11:07:43Z AlbrechtS $".
+ * End of "$Id: flstring.h 11094 2016-01-31 02:49:56Z AlbrechtS $".
  */
